@@ -1,14 +1,21 @@
 from copy import copy
-from threading import Thread
 
 from pm4py import util as pmutil
-from pm4py.algo.filtering.log.variants import variants_filter as variants_module
-from pm4py.objects.log.util import xes as xes_util
+from pm4py.statistics.variants.log import get as variants_module
+from pm4py.util import xes_constants as xes_util
 from pm4py.objects.petri import semantics
+from pm4py.objects.petri.petrinet import Marking
 from pm4py.objects.petri.utils import get_places_shortest_path_by_hidden, get_s_components_from_petri
 from pm4py.util import constants
+from pm4py.objects.log import log as log_implementation
+from pm4py.objects.petri.importer.versions import pnml as petri_importer
+from pm4py.objects.petri import align_utils
+from copy import deepcopy, copy
 
-MAX_REC_DEPTH = 18
+PARAMETER_VARIANT_DELIMITER = "variant_delimiter"
+DEFAULT_VARIANT_DELIMITER = ","
+
+MAX_REC_DEPTH = 50
 MAX_IT_FINAL1 = 5
 MAX_IT_FINAL2 = 5
 MAX_REC_DEPTH_HIDTRANSENABL = 2
@@ -46,8 +53,8 @@ def add_missing_tokens(t, marking):
     for a in t.in_arcs:
         if marking[a.source] < a.weight:
             missing = missing + (a.weight - marking[a.source])
-            marking[a.source] = marking[a.source] + a.weight
             tokens_added[a.source] = a.weight - marking[a.source]
+            marking[a.source] = marking[a.source] + a.weight
     return [missing, tokens_added]
 
 
@@ -61,9 +68,11 @@ def get_consumed_tokens(t):
         Transition that should be enabled
     """
     consumed = 0
+    consumed_map = {}
     for a in t.in_arcs:
         consumed = consumed + a.weight
-    return consumed
+        consumed_map[a.source] = a.weight
+    return consumed, consumed_map
 
 
 def get_produced_tokens(t):
@@ -76,9 +85,11 @@ def get_produced_tokens(t):
         Transition that should be enabled
     """
     produced = 0
+    produced_map = {}
     for a in t.out_arcs:
         produced = produced + a.weight
-    return produced
+        produced_map[a.target] = a.weight
+    return produced, produced_map
 
 
 def merge_dicts(x, y):
@@ -100,35 +111,6 @@ def merge_dicts(x, y):
                 x[key] = y[key]
 
 
-def get_hidden_trans_reached_trans(t, net, rec_depth):
-    """
-    Get visible transitions reachable by enabling a hidden transition
-
-    Parameters
-    ----------
-    t
-        Transition that should be enabled
-    net
-        Petri net
-    rec_depth
-        Current recursion depth
-    """
-    reach_trans = {}
-    if rec_depth > MAX_REC_DEPTH:
-        return reach_trans
-    # if rec_depth > DebugConst.REACH_MRD:
-    #    DebugConst.REACH_MRD = rec_depth
-    for a1 in t.out_arcs:
-        place = a1.target
-        for a2 in place.out_arcs:
-            t2 = a2.target
-            if t2.label is not None:
-                reach_trans[t2.label] = rec_depth
-            if t2.label is None:
-                merge_dicts(reach_trans, get_hidden_trans_reached_trans(t2, net, rec_depth + 1))
-    return reach_trans
-
-
 def get_places_with_missing_tokens(t, marking):
     """
     Get places with missing tokens
@@ -327,47 +309,6 @@ def apply_hidden_trans(t, net, marking, places_shortest_paths_by_hidden, act_tr,
     return [net, marking, act_tr, vis_mark]
 
 
-def get_visible_transitions_eventually_enabled_by_marking(net, marking):
-    """
-    Get visible transitions eventually enabled by marking (passing possibly through hidden transitions)
-
-    Parameters
-    ----------
-    net
-        Petri net
-    marking
-        Current marking
-    """
-    all_enabled_transitions = list(semantics.enabled_transitions(net, marking))
-    initial_all_enabled_transitions_marking_dictio = {}
-    all_enabled_transitions_marking_dictio = {}
-    for trans in all_enabled_transitions:
-        all_enabled_transitions_marking_dictio[trans] = marking
-        initial_all_enabled_transitions_marking_dictio[trans] = marking
-    visible_transitions = set()
-    visited_transitions = set()
-
-    i = 0
-    while i < len(all_enabled_transitions):
-        t = all_enabled_transitions[i]
-        marking_copy = copy(all_enabled_transitions_marking_dictio[t])
-
-        if repr([t, marking_copy]) not in visited_transitions:
-            if t.label is not None:
-                visible_transitions.add(t)
-            else:
-                if semantics.is_enabled(t, net, marking_copy):
-                    new_marking = semantics.execute(t, net, marking_copy)
-                    new_enabled_transitions = list(semantics.enabled_transitions(net, new_marking))
-                    for t2 in new_enabled_transitions:
-                        all_enabled_transitions.append(t2)
-                        all_enabled_transitions_marking_dictio[t2] = new_marking
-            visited_transitions.add(repr([t, marking_copy]))
-        i = i + 1
-
-    return visible_transitions
-
-
 def break_condition_final_marking(marking, final_marking):
     """
     Verify break condition for final marking
@@ -394,7 +335,7 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                 walk_through_hidden_trans=True, post_fix_caching=None,
                 marking_to_activity_caching=None, is_reduction=False, thread_maximum_ex_time=MAX_DEF_THR_EX_TIME,
                 enable_postfix_cache=False, enable_marktoact_cache=False, cleaning_token_flood=False,
-                s_components=None):
+                s_components=None, trace_occurrences=1):
     """
     Apply the token replaying algorithm to a trace
 
@@ -411,7 +352,7 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
     trans_map
         Map between transitions labels and transitions
     enable_pltr_fitness
-        Enable fitness calculation at place/transition level
+        Enable fitness retrieval at place/transition level
     place_fitness
         Current dictionary of places associated with unfit traces
     transition_fitness
@@ -501,13 +442,27 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                                                                               marking):
                         visited_transitions = set()
                         prev_len_activated_transitions = len(act_trans)
-                        [net, marking, act_trans, vis_mark] = apply_hidden_trans(t, net,
-                                                                                 marking,
-                                                                                 places_shortest_path_by_hidden,
-                                                                                 act_trans,
-                                                                                 0,
-                                                                                 visited_transitions,
-                                                                                 vis_mark)
+                        [net, new_marking, new_act_trans, new_vis_mark] = apply_hidden_trans(t, net,
+                                                                                             copy(marking),
+                                                                                             places_shortest_path_by_hidden,
+                                                                                             copy(act_trans),
+                                                                                             0,
+                                                                                             copy(visited_transitions),
+                                                                                             copy(vis_mark))
+                        for jj5 in range(len(act_trans), len(new_act_trans)):
+                            tt5 = new_act_trans[jj5]
+                            c, cmap = get_consumed_tokens(tt5)
+                            p, pmap = get_produced_tokens(tt5)
+                            if enable_pltr_fitness:
+                                for pl2 in cmap:
+                                    if pl2 in place_fitness:
+                                        place_fitness[pl2]["c"] += cmap[pl2] * trace_occurrences
+                                for pl2 in pmap:
+                                    if pl2 in place_fitness:
+                                        place_fitness[pl2]["p"] += pmap[pl2] * trace_occurrences
+                            consumed = consumed + c
+                            produced = produced + p
+                        marking, act_trans, vis_mark = new_marking, new_act_trans, new_vis_mark
                     is_initially_enabled = True
                     old_marking_names = [x.name for x in list(marking.keys())]
                     if not semantics.is_enabled(t, net, marking):
@@ -522,6 +477,7 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                             for place in tokens_added.keys():
                                 if place in place_fitness:
                                     place_fitness[place]["underfed_traces"].add(trace)
+                                place_fitness[place]["m"] += tokens_added[place]
                             if trace not in transition_fitness[t]["underfed_traces"]:
                                 transition_fitness[t]["underfed_traces"][trace] = list()
                             transition_fitness[t]["underfed_traces"][trace].append(current_event_map)
@@ -529,10 +485,17 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                         if trace not in transition_fitness[t]["fit_traces"]:
                             transition_fitness[t]["fit_traces"][trace] = list()
                         transition_fitness[t]["fit_traces"][trace].append(current_event_map)
-                    c = get_consumed_tokens(t)
-                    p = get_produced_tokens(t)
+                    c, cmap = get_consumed_tokens(t)
+                    p, pmap = get_produced_tokens(t)
                     consumed = consumed + c
                     produced = produced + p
+                    if enable_pltr_fitness:
+                        for pl2 in cmap:
+                            if pl2 in place_fitness:
+                                place_fitness[pl2]["c"] += cmap[pl2] * trace_occurrences
+                        for pl2 in pmap:
+                            if pl2 in place_fitness:
+                                place_fitness[pl2]["p"] += pmap[pl2] * trace_occurrences
                     if semantics.is_enabled(t, net, marking):
                         marking = semantics.execute(t, net, marking)
                         act_trans.append(t)
@@ -583,14 +546,22 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                             marking = semantics.execute(t, net, marking)
                             act_trans.append(t)
                             vis_mark.append(marking)
+                            c, cmap = get_consumed_tokens(t)
+                            p, pmap = get_produced_tokens(t)
+                            if enable_pltr_fitness:
+                                for pl2 in cmap:
+                                    if pl2 in place_fitness:
+                                        place_fitness[pl2]["c"] += cmap[pl2] * trace_occurrences
+                                for pl2 in pmap:
+                                    if pl2 in place_fitness:
+                                        place_fitness[pl2]["p"] += pmap[pl2] * trace_occurrences
+                            consumed = consumed + c
+                            produced = produced + p
                     if break_condition_final_marking(marking, final_marking):
                         break
             else:
                 break
 
-        # if i > DebugConst.REACH_ITF1:
-        #    DebugConst.REACH_ITF1 = i
-
         # try to reach the final marking in a different fashion, if not already reached
         if not break_condition_final_marking(marking, final_marking):
             if len(final_marking) == 1:
@@ -609,19 +580,33 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                             if semantics.is_enabled(t, net, marking):
                                 marking = semantics.execute(t, net, marking)
                                 act_trans.append(t)
+                                c, cmap = get_consumed_tokens(t)
+                                p, pmap = get_produced_tokens(t)
+                                if enable_pltr_fitness:
+                                    for pl2 in cmap:
+                                        if pl2 in place_fitness:
+                                            place_fitness[pl2]["c"] += cmap[pl2] * trace_occurrences
+                                    for pl2 in pmap:
+                                        if pl2 in place_fitness:
+                                            place_fitness[pl2]["p"] += pmap[pl2] * trace_occurrences
+                                consumed = consumed + c
+                                produced = produced + p
                                 vis_mark.append(marking)
                                 continue
                             else:
                                 break
 
-                # if i > DebugConst.REACH_ITF2:
-                #    DebugConst.REACH_ITF2 = i
-
-    if break_condition_final_marking(marking, final_marking):
-        consumed = consumed + sum_tokens_fm
-
     marking_before_cleaning = copy(marking)
 
+    # 25/02/2020: fix to the missing tokens mark (if the final marking is not reached)
+    # (not inficiating the previous stat)
+    diff_fin_mark_mark = Marking()
+    for p in final_marking:
+        diff = final_marking[p] - marking[p]
+        if diff > 0:
+            diff_fin_mark_mark[p] = diff
+
+    # set up the remaining tokens count
     remaining = 0
     for p in marking:
         if p in final_marking:
@@ -631,6 +616,13 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                     if p in place_fitness:
                         if trace not in place_fitness[p]["underfed_traces"]:
                             place_fitness[p]["overfed_traces"].add(trace)
+                        place_fitness[p]["r"] += marking[p] * trace_occurrences
+        # 25/02/2020: added missing part to statistics
+        elif enable_pltr_fitness:
+            if p in place_fitness:
+                if trace not in place_fitness[p]["underfed_traces"]:
+                    place_fitness[p]["overfed_traces"].add(trace)
+                place_fitness[p]["r"] += marking[p] * trace_occurrences
         remaining = remaining + marking[p]
 
     for p in current_remaining_map:
@@ -638,6 +630,7 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
             if p in place_fitness:
                 if trace not in place_fitness[p]["underfed_traces"] and trace not in place_fitness[p]["overfed_traces"]:
                     place_fitness[p]["overfed_traces"].add(trace)
+                place_fitness[p]["r"] += current_remaining_map[p] * trace_occurrences
         remaining = remaining + current_remaining_map[p]
 
     if consider_remaining_in_fitness:
@@ -645,9 +638,24 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
     else:
         is_fit = (missing == 0)
 
+    # separate global counts from local statistics in the case these are not enabled by the options
+    for pl in final_marking:
+        consumed += final_marking[pl]
+    # 25/02/2020: update the missing tokens count here
+    for pl in diff_fin_mark_mark:
+        missing += diff_fin_mark_mark[pl]
+
+    if enable_pltr_fitness:
+        for pl in initial_marking:
+            place_fitness[pl]["p"] += initial_marking[pl] * trace_occurrences
+        for pl in final_marking:
+            place_fitness[pl]["c"] += final_marking[pl] * trace_occurrences
+        for pl in diff_fin_mark_mark:
+            place_fitness[pl]["m"] += diff_fin_mark_mark[pl] * trace_occurrences
+
     if consumed > 0 and produced > 0:
-        #trace_fitness = (1.0 - float(missing) / float(consumed)) * (1.0 - float(remaining) / float(produced))
-        trace_fitness = 0.5 * (1.0 - float(missing) / float(consumed)) + 0.5 * (1.0 - float(remaining) / float(produced))
+        trace_fitness = 0.5 * (1.0 - float(missing) / float(consumed)) + 0.5 * (
+                1.0 - float(remaining) / float(produced))
     else:
         trace_fitness = 1.0
 
@@ -682,18 +690,19 @@ def apply_trace(trace, net, initial_marking, final_marking, trans_map, enable_pl
                             "previousActivity": previous_activity}
 
     return [is_fit, trace_fitness, act_trans, transitions_with_problems, marking_before_cleaning,
-            get_visible_transitions_eventually_enabled_by_marking(net, marking_before_cleaning), missing, consumed,
+            align_utils.get_visible_transitions_eventually_enabled_by_marking(net, marking_before_cleaning), missing,
+            consumed,
             remaining, produced]
 
 
-class ApplyTraceTokenReplay(Thread):
+class ApplyTraceTokenReplay:
     def __init__(self, trace, net, initial_marking, final_marking, trans_map, enable_pltr_fitness, place_fitness,
                  transition_fitness, notexisting_activities_in_model,
                  places_shortest_path_by_hidden, consider_remaining_in_fitness, activity_key="concept:name",
                  reach_mark_through_hidden=True, stop_immediately_when_unfit=False,
                  walk_through_hidden_trans=True, post_fix_caching=None,
                  marking_to_activity_caching=None, is_reduction=False, thread_maximum_ex_time=MAX_DEF_THR_EX_TIME,
-                 cleaning_token_flood=False, s_components=None):
+                 cleaning_token_flood=False, s_components=None, trace_occurrences=1):
         """
         Constructor
 
@@ -706,7 +715,7 @@ class ApplyTraceTokenReplay(Thread):
         trans_map
             Map between transitions labels and transitions
         enable_pltr_fitness
-            Enable fitness calculation at place/transition level
+            Enable fitness retrieval at place/transition level
         place_fitness
             Current dictionary of places associated with unfit traces
         transition_fitness
@@ -738,6 +747,8 @@ class ApplyTraceTokenReplay(Thread):
             Decides if a cleaning of the token flood shall be operated
         s_components
             S-components of the Petri net
+        trace_occurrences
+            Trace weight (number of occurrences)
         """
         self.thread_is_alive = True
         self.trace = trace
@@ -776,8 +787,7 @@ class ApplyTraceTokenReplay(Thread):
         self.remaining = None
         self.produced = None
         self.s_components = s_components
-
-        Thread.__init__(self)
+        self.trace_occurrences = trace_occurrences
 
     def run(self):
         """
@@ -799,7 +809,8 @@ class ApplyTraceTokenReplay(Thread):
                         enable_postfix_cache=self.enable_postfix_cache,
                         enable_marktoact_cache=self.enable_marktoact_cache,
                         cleaning_token_flood=self.cleaning_token_flood,
-                        s_components=self.s_components)
+                        s_components=self.s_components,
+                        trace_occurrences=self.trace_occurrences)
         self.thread_is_alive = False
 
 
@@ -823,58 +834,6 @@ class MarkingToActivityCaching:
         self.cache = {}
 
 
-def check_threads(net, threads, threads_results, all_activated_transitions, is_reduction=False):
-    """
-    Check threads aliveness and terminate them accordingly
-
-    Parameters
-    ------------
-    net
-        Petri net
-    threads
-        Current opened threads
-    threads_results
-        Threads execution result (to be returned)
-    all_activated_transitions
-        List of activated transitions during the replay (to be returned)
-    is_reduction
-        Is this a reduction replay (boolean)
-
-    Returns
-    ------------
-    threads
-        Current opened threads
-    threads_results
-        Threads execution result
-    all_activated_transitions
-        List of activated transitions during the replay
-    """
-    threads_keys = list(threads.keys())
-    terminated_threads_keys = [tk for tk in threads_keys if threads[tk].thread_is_alive is False]
-    for tk in terminated_threads_keys:
-        t = threads[tk]
-        threads_results[tk] = {"trace_is_fit": copy(t.t_fit),
-                               "trace_fitness": copy(t.t_value),
-                               "activated_transitions": copy(t.act_trans),
-                               "reached_marking": copy(t.reached_marking),
-                               "enabled_transitions_in_marking": copy(
-                                   t.enabled_trans_in_mark),
-                               "transitions_with_problems": copy(
-                                   t.trans_probl),
-                               "missing_tokens": t.missing,
-                               "consumed_tokens": t.consumed,
-                               "remaining_tokens": t.remaining,
-                               "produced_tokens": t.produced}
-        all_activated_transitions.update(set(t.act_trans))
-
-        del threads_keys[threads_keys.index(tk)]
-        del threads[tk]
-
-        if is_reduction and len(all_activated_transitions) == len(net.transitions):
-            break
-    return threads, threads_results, all_activated_transitions
-
-
 def get_variant_from_trace(trace, activity_key, disable_variants=False):
     """
     Gets the variant from the trace (allow disabling)
@@ -930,7 +889,7 @@ def apply_log(log, net, initial_marking, final_marking, enable_pltr_fitness=Fals
               activity_key="concept:name", reach_mark_through_hidden=True, stop_immediately_unfit=False,
               walk_through_hidden_trans=True, places_shortest_path_by_hidden=None,
               variants=None, is_reduction=False, thread_maximum_ex_time=MAX_DEF_THR_EX_TIME,
-              cleaning_token_flood=False, disable_variants=False):
+              cleaning_token_flood=False, disable_variants=False, return_object_names=False):
     """
     Apply token-based replay to a log
 
@@ -945,7 +904,7 @@ def apply_log(log, net, initial_marking, final_marking, enable_pltr_fitness=Fals
     final_marking
         Final marking
     enable_pltr_fitness
-        Enable fitness calculation at place level
+        Enable fitness retrieval at place level
     consider_remaining_in_fitness
         Boolean value telling if the remaining tokens should be considered in fitness evaluation
     activity_key
@@ -968,11 +927,13 @@ def apply_log(log, net, initial_marking, final_marking, enable_pltr_fitness=Fals
         Decides if a cleaning of the token flood shall be operated
     disable_variants
         Disable variants grouping
+    return_object_names
+        Decides whether names instead of object pointers shall be returned
     """
     post_fix_cache = PostFixCaching()
     marking_to_activity_cache = MarkingToActivityCaching()
     if places_shortest_path_by_hidden is None:
-        places_shortest_path_by_hidden = get_places_shortest_path_by_hidden(net)
+        places_shortest_path_by_hidden = get_places_shortest_path_by_hidden(net, MAX_REC_DEPTH)
 
     place_fitness_per_trace = {}
     transition_fitness_per_trace = {}
@@ -981,7 +942,8 @@ def apply_log(log, net, initial_marking, final_marking, enable_pltr_fitness=Fals
 
     if enable_pltr_fitness:
         for place in net.places:
-            place_fitness_per_trace[place] = {"underfed_traces": set(), "overfed_traces": set()}
+            place_fitness_per_trace[place] = {"underfed_traces": set(), "overfed_traces": set(), "m": 0, "r": 0, "c": 0,
+                                              "p": 0}
         for transition in net.transitions:
             if transition.label:
                 transition_fitness_per_trace[transition] = {"underfed_traces": {}, "fit_traces": {}}
@@ -1008,20 +970,6 @@ def apply_log(log, net, initial_marking, final_marking, enable_pltr_fitness=Fals
 
                 for i in range(len(vc)):
                     variant = vc[i][0]
-                    threads_keys = list(threads.keys())
-                    while len(threads_keys) > MAX_NO_THREADS:
-                        threads, threads_results, all_activated_transitions = check_threads(net, threads,
-                                                                                            threads_results,
-                                                                                            all_activated_transitions,
-                                                                                            is_reduction=is_reduction)
-                        if is_reduction and len(all_activated_transitions) == len(net.transitions):
-                            break
-                        threads_keys = list(threads.keys())
-                    threads, threads_results, all_activated_transitions = check_threads(net, threads, threads_results,
-                                                                                        all_activated_transitions,
-                                                                                        is_reduction=is_reduction)
-                    if is_reduction and len(all_activated_transitions) == len(net.transitions):
-                        break
                     threads[variant] = ApplyTraceTokenReplay(variants[variant][0], net, initial_marking, final_marking,
                                                              trans_map, enable_pltr_fitness, place_fitness_per_trace,
                                                              transition_fitness_per_trace,
@@ -1037,20 +985,38 @@ def apply_log(log, net, initial_marking, final_marking, enable_pltr_fitness=Fals
                                                              is_reduction=is_reduction,
                                                              thread_maximum_ex_time=thread_maximum_ex_time,
                                                              cleaning_token_flood=cleaning_token_flood,
-                                                             s_components=s_components)
-                    threads[variant].start()
-                while len(threads) > 0:
-                    threads_keys = list(threads.keys())
-                    t = threads[threads_keys[0]]
-                    t.join()
-                    threads, threads_results, all_activated_transitions = check_threads(net, threads, threads_results,
-                                                                                        all_activated_transitions,
-                                                                                        is_reduction=is_reduction)
-                    if is_reduction and len(all_activated_transitions) == len(net.transitions):
-                        break
+                                                             s_components=s_components, trace_occurrences=vc[i][1])
+                    threads[variant].run()
+                    t = threads[variant]
+                    threads_results[variant] = {"trace_is_fit": copy(t.t_fit),
+                                                "trace_fitness": float(copy(t.t_value)),
+                                                "activated_transitions": copy(t.act_trans),
+                                                "reached_marking": copy(t.reached_marking),
+                                                "enabled_transitions_in_marking": copy(
+                                                    t.enabled_trans_in_mark),
+                                                "transitions_with_problems": copy(
+                                                    t.trans_probl),
+                                                "missing_tokens": int(t.missing),
+                                                "consumed_tokens": int(t.consumed),
+                                                "remaining_tokens": int(t.remaining),
+                                                "produced_tokens": int(t.produced)}
 
+                    if return_object_names:
+                        threads_results[variant]["activated_transitions_labels"] = [x.label for x in
+                                                                        threads_results[variant]["activated_transitions"]]
+                        threads_results[variant]["activated_transitions"] = [x.name for x in threads_results[variant]["activated_transitions"]]
+                        threads_results[variant]["enabled_transitions_in_marking_labels"] = [x.label for x in threads_results[variant][
+                            "enabled_transitions_in_marking"]]
+                        threads_results[variant]["enabled_transitions_in_marking"] = [x.name for x in threads_results[variant][
+                            "enabled_transitions_in_marking"]]
+                        threads_results[variant]["transitions_with_problems"] = [x.name for x in
+                                                                                 threads_results[variant][
+                                                                                     "transitions_with_problems"]]
+                        threads_results[variant]["reached_marking"] = {x.name: y for x, y in
+                                                                       threads_results[variant][
+                                                                           "reached_marking"].items()}
+                    del threads[variant]
                 for trace in log:
-                    # trace_variant = ",".join([x[activity_key] for x in trace])
                     trace_variant = get_variant_from_trace(trace, activity_key, disable_variants=disable_variants)
                     if trace_variant in threads_results:
                         t = threads_results[trace_variant]
@@ -1085,13 +1051,15 @@ def apply(log, net, initial_marking, final_marking, parameters=None):
         parameters = {}
 
     enable_pltr_fitness = False
-    consider_remaining_in_fitness = False
+    # changed default to uniform behavior with token-based replay fitness
+    consider_remaining_in_fitness = True
     try_to_reach_final_marking_through_hidden = True
     stop_immediately_unfit = False
     walk_through_hidden_trans = True
     is_reduction = False
     cleaning_token_flood = False
     disable_variants = False
+    return_names = False
     thread_maximum_ex_time = MAX_DEF_THR_EX_TIME
     places_shortest_path_by_hidden = None
     activity_key = xes_util.DEFAULT_NAME_KEY
@@ -1107,6 +1075,8 @@ def apply(log, net, initial_marking, final_marking, parameters=None):
         try_to_reach_final_marking_through_hidden = parameters["try_to_reach_final_marking_through_hidden"]
     if "stop_immediately_unfit" in parameters:
         stop_immediately_unfit = parameters["stop_immediately_unfit"]
+    if "return_names" in parameters:
+        return_names = parameters["return_names"]
     if "walk_through_hidden_trans" in parameters:
         walk_through_hidden_trans = parameters[
             "walk_through_hidden_trans"]
@@ -1132,4 +1102,53 @@ def apply(log, net, initial_marking, final_marking, parameters=None):
                      walk_through_hidden_trans=walk_through_hidden_trans,
                      places_shortest_path_by_hidden=places_shortest_path_by_hidden, activity_key=activity_key,
                      variants=variants, is_reduction=is_reduction, thread_maximum_ex_time=thread_maximum_ex_time,
-                     cleaning_token_flood=cleaning_token_flood, disable_variants=disable_variants)
+                     cleaning_token_flood=cleaning_token_flood, disable_variants=disable_variants,
+                     return_object_names=return_names)
+
+
+def apply_variants_list(variants_list, net, initial_marking, final_marking, parameters=None):
+    if parameters is None:
+        parameters = {}
+    parameters["return_names"] = True
+
+    activity_key = parameters[
+        pmutil.constants.PARAMETER_CONSTANT_ACTIVITY_KEY] if pmutil.constants.PARAMETER_CONSTANT_ACTIVITY_KEY in parameters else xes_util.DEFAULT_NAME_KEY
+    variant_delimiter = parameters[
+        PARAMETER_VARIANT_DELIMITER] if PARAMETER_VARIANT_DELIMITER in parameters else DEFAULT_VARIANT_DELIMITER
+
+    log = log_implementation.EventLog()
+    for var_item in variants_list:
+        variant = var_item[0].split(variant_delimiter)
+        trace = log_implementation.Trace()
+        for activ in variant:
+            event = log_implementation.Event({activity_key: activ})
+            trace.append(event)
+        log.append(trace)
+
+    return apply(log, net, initial_marking, final_marking, parameters=parameters)
+
+
+def apply_variants_dictionary(variants, net, initial_marking, final_marking, parameters=None):
+    if parameters is None:
+        parameters = {}
+
+    var_list = {x: len(y) for x, y in variants.items()}
+    return apply_variants_list(var_list, net, initial_marking, final_marking, parameters=parameters)
+
+
+def apply_variants_list_petri_string(variants_list, petri_string, parameters=None):
+    if parameters is None:
+        parameters = {}
+
+    net, im, fm = petri_importer.import_petri_from_string(petri_string, parameters=parameters)
+
+    return apply_variants_list(variants_list, net, im, fm, parameters=parameters)
+
+
+def apply_variants_list_petri_string_multiprocessing(output, variants_list, petri_string, parameters=None):
+    if parameters is None:
+        parameters = {}
+
+    ret = apply_variants_list_petri_string(variants_list, petri_string, parameters=parameters)
+
+    output.put(ret)
