@@ -3,7 +3,7 @@ import time
 from pm4py import util as pmutil
 from pm4py.objects import petri
 from pm4py.objects.log.log import EventLog, Trace
-from pm4py.objects.log.util import xes as xes_util
+from pm4py.util import xes_constants as xes_util
 from pm4py.objects.petri.petrinet import Marking
 
 REMOVE_UNCONNECTED = "remove_unconnected"
@@ -322,7 +322,7 @@ def processing(log, causal, follows, parameters=None):
     net = petri.petrinet.PetriNet('alpha_plus_net_' + str(time.time()))
     label_transition_dict = {}
     for label in labels:
-        if label is not 'artificial_start' and label is not 'artificial_end':
+        if label != 'artificial_start' and label != 'artificial_end':
             label_transition_dict[label] = petri.petrinet.PetriNet.Transition(label, label)
             net.transitions.add(label_transition_dict[label])
         else:
