@@ -1,9 +1,8 @@
+from pm4py.statistics.variants.log.get import get_variants_from_log_trace_idx, get_variants, get_variants_along_with_case_durations, get_variants_sorted_by_count, convert_variants_trace_idx_to_trace_obj
 from pm4py.algo.filtering.common import filtering_constants
 from pm4py.objects.log.log import EventLog
-from pm4py.objects.log.util.xes import DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY
-from pm4py.util.constants import PARAMETER_CONSTANT_ACTIVITY_KEY, PARAMETER_CONSTANT_TIMESTAMP_KEY
-
-import numpy as np
+from pm4py.util.xes_constants import DEFAULT_NAME_KEY
+from pm4py.util.constants import PARAMETER_CONSTANT_ACTIVITY_KEY
 
 
 def apply(log, admitted_variants, parameters=None):
@@ -34,153 +33,33 @@ def apply(log, admitted_variants, parameters=None):
     return log
 
 
-def get_variants(log, parameters=None):
+def filter_log_variants_percentage(log, percentage=0.8, parameters=None):
     """
-    Gets a dictionary whose key is the variant and as value there
-    is the list of traces that share the variant
+    Filters a log by variants percentage
 
     Parameters
-    ----------
+    -------------
     log
-        Trace log
+        Event log
+    percentage
+        Percentage
     parameters
-        Parameters of the algorithm, including:
-            activity_key -> Attribute identifying the activity in the log
+        Parameters of the algorithm
 
     Returns
-    ----------
-    variant
-        Dictionary with variant as the key and the list of traces as the value
-    """
-
-    variants_trace_idx = get_variants_from_log_trace_idx(log, parameters=parameters)
-
-    all_var = convert_variants_trace_idx_to_trace_obj(log, variants_trace_idx)
-
-    return all_var
-
-
-def get_variants_along_with_case_durations(log, parameters=None):
-    """
-    Gets a dictionary whose key is the variant and as value there
-    is the list of traces that share the variant
-
-    Parameters
-    ----------
-    log
-        Trace log
-    parameters
-        Parameters of the algorithm, including:
-            activity_key -> Attribute identifying the activity in the log
-
-    Returns
-    ----------
-    variant
-        Dictionary with variant as the key and the list of traces as the value
+    -------------
+    filtered_log
+        Filtered log (by variants percentage)
     """
     if parameters is None:
         parameters = {}
 
-    timestamp_key = parameters[
-        PARAMETER_CONSTANT_TIMESTAMP] if PARAMETER_CONSTANT_TIMESTAMP_KEY in parameters else DEFAULT_TIMESTAMP_KEY
+    variants = get_variants(log, parameters=parameters)
 
-    variants_trace_idx = get_variants_from_log_trace_idx(log, parameters=parameters)
-    all_var = convert_variants_trace_idx_to_trace_obj(log, variants_trace_idx)
-
-    all_durations = {}
-
-    for var in all_var:
-        all_durations[var] = []
-        for trace in all_var[var]:
-            all_durations[var].append((trace[-1][timestamp_key] - trace[0][timestamp_key]).total_seconds())
-        all_durations[var] = np.array(all_durations[var])
-
-    return all_var, all_durations
+    return filter_variants_variants_percentage(log, variants, variants_percentage=percentage)
 
 
-def get_variants_from_log_trace_idx(log, parameters=None):
-    """
-    Gets a dictionary whose key is the variant and as value there
-    is the list of traces indexes that share the variant
-
-    Parameters
-    ----------
-    log
-        Log
-    parameters
-        Parameters of the algorithm, including:
-            activity_key -> Attribute identifying the activity in the log
-
-    Returns
-    ----------
-    variant
-        Dictionary with variant as the key and the list of traces indexes as the value
-    """
-    if parameters is None:
-        parameters = {}
-
-    attribute_key = parameters[
-        PARAMETER_CONSTANT_ACTIVITY_KEY] if PARAMETER_CONSTANT_ACTIVITY_KEY in parameters else DEFAULT_NAME_KEY
-
-    variants = {}
-    for trace_idx, trace in enumerate(log):
-        variant = ",".join([x[attribute_key] for x in trace if attribute_key in x])
-        if variant not in variants:
-            variants[variant] = []
-        variants[variant].append(trace_idx)
-
-    return variants
-
-
-def convert_variants_trace_idx_to_trace_obj(log, variants_trace_idx):
-    """
-    Converts variants expressed as trace indexes to trace objects
-
-    Parameters
-    -----------
-    log
-        Trace log object
-    variants_trace_idx
-        Variants associated to a list of belonging indexes
-
-    Returns
-    -----------
-    variants
-        Variants associated to a list of belonging traces
-    """
-    variants = {}
-
-    for key in variants_trace_idx:
-        variants[key] = []
-        for value in variants_trace_idx[key]:
-            variants[key].append(log[value])
-
-    return variants
-
-
-def get_variants_sorted_by_count(variants):
-    """
-    From the dictionary of variants returns an ordered list of variants
-    along with their count
-
-    Parameters
-    ----------
-    variants
-        Dictionary with variant as the key and the list of traces as the value
-
-    Returns
-    ----------
-    var_count
-        List of variant names along with their count
-    """
-    var_count = []
-    for variant in variants:
-        var_count.append([variant, len(variants[variant])])
-    var_count = sorted(var_count, key=lambda x: x[1], reverse=True)
-    return var_count
-
-
-def filter_log_by_variants_percentage(log, variants, variants_percentage=0.0):
+def filter_variants_variants_percentage(log, variants, variants_percentage=0.0):
     """
     Filter the log by variants percentage
 
@@ -202,15 +81,19 @@ def filter_log_by_variants_percentage(log, variants, variants_percentage=0.0):
     no_of_traces = len(log)
     variant_count = get_variants_sorted_by_count(variants)
     already_added_sum = 0
+    shall_break_under = -1
 
     for i in range(len(variant_count)):
         variant = variant_count[i][0]
         varcount = variant_count[i][1]
+        if varcount < shall_break_under:
+            break
         percentage_already_added = already_added_sum / no_of_traces
-        if already_added_sum == 0 or percentage_already_added < variants_percentage:
-            for trace in variants[variant]:
-                filtered_log.append(trace)
-            already_added_sum = already_added_sum + varcount
+        for trace in variants[variant]:
+            filtered_log.append(trace)
+        already_added_sum = already_added_sum + varcount
+        if percentage_already_added >= variants_percentage:
+            shall_break_under = varcount
 
     return filtered_log
 
@@ -288,5 +171,5 @@ def apply_auto_filter(log, variants=None, parameters=None):
     if variants is None:
         variants = get_variants(log, parameters=parameters_variants)
     variants_percentage = find_auto_threshold(log, variants, decreasing_factor)
-    filtered_log = filter_log_by_variants_percentage(log, variants, variants_percentage)
+    filtered_log = filter_variants_variants_percentage(log, variants, variants_percentage)
     return filtered_log
