@@ -1,8 +1,10 @@
 from pm4py.objects.conversion.process_tree.versions import to_petri_net
+from pm4py.objects.conversion.process_tree.versions import to_petri_net_transition_bordered
 
-TO_PETRI_NET = "to_petri_net"
+TO_PETRI_NET = "to_petri_net" # old / legacy code
+TO_PETRI_NET_TRANSITION_BORDERED = "to_petri_net_transition_bordered"
 
-VERSIONS = {TO_PETRI_NET: to_petri_net.apply}
+VERSIONS = {TO_PETRI_NET: to_petri_net.apply, TO_PETRI_NET_TRANSITION_BORDERED: to_petri_net_transition_bordered.apply}
 
 
 def apply(tree, parameters=None, variant=TO_PETRI_NET):
