@@ -1,13 +1,30 @@
 from copy import copy, deepcopy
+import time
+import random
 
 import networkx as nx
 
 from pm4py.objects import petri
-from pm4py.objects.log.util import xes as xes_util
+from pm4py.objects.log.log import Trace, Event
+from pm4py.util import xes_constants as xes_util
 from pm4py.objects.petri.check_soundness import check_petri_wfnet_and_soundness
 from pm4py.objects.petri.networkx_graph import create_networkx_directed_graph
 
 
+def pre_set(elem):
+    pre = set()
+    for a in elem.in_arcs:
+        pre.add(a.source)
+    return pre
+
+
+def post_set(elem):
+    post = set()
+    for a in elem.out_arcs:
+        post.add(a.target)
+    return post
+
+
 def remove_transition(net, trans):
     """
     Remove a transition from a Petri net
@@ -39,6 +56,32 @@ def remove_transition(net, trans):
     return net
 
 
+def add_place(net, name=None):
+    name = name if name is not None else 'p_' + str(len(net.places)) + '_' + str(time.time()) + str(
+        random.randint(0, 10000))
+    p = petri.petrinet.PetriNet.Place(name=name)
+    net.places.add(p)
+    return p
+
+
+def add_transition(net, name=None, label=None):
+    name = name if name is not None else 't_' + str(len(net.transitions)) + '_' + str(time.time()) + str(
+        random.randint(0, 10000))
+    t = petri.petrinet.PetriNet.Transition(name=name, label=label)
+    net.transitions.add(t)
+    return t
+
+
+def merge(trgt=None, nets=None):
+    trgt = trgt if trgt is not None else petri.petrinet.PetriNet()
+    nets = nets if nets is not None else list()
+    for net in nets:
+        trgt.transitions.update(net.transitions)
+        trgt.places.update(net.places)
+        trgt.arcs.update(net.arcs)
+    return trgt
+
+
 def remove_place(net, place):
     """
     Remove a place from a Petri net
@@ -125,7 +168,7 @@ def construct_trace_net(trace, trace_name_key=xes_util.DEFAULT_NAME_KEY, activit
 def construct_trace_net_cost_aware(trace, costs, trace_name_key=xes_util.DEFAULT_NAME_KEY,
                                    activity_key=xes_util.DEFAULT_NAME_KEY):
     """
-    Creates a trace net, i.e. a trace in Petri net form.
+    Creates a trace net, i.e. a trace in Petri net form mapping specific costs to transitions.
 
     Parameters
     ----------
@@ -136,7 +179,8 @@ def construct_trace_net_cost_aware(trace, costs, trace_name_key=xes_util.DEFAULT
 
     Returns
     -------
-    tuple: :class:`tuple` of the net, initial marking and the final marking
+    tuple: :class:`tuple` of the net, initial marking, final marking and map of costs
+
 
     """
     net = petri.petrinet.PetriNet(
@@ -155,10 +199,12 @@ def construct_trace_net_cost_aware(trace, costs, trace_name_key=xes_util.DEFAULT
     return net, petri.petrinet.Marking({place_map[0]: 1}), petri.petrinet.Marking({place_map[len(trace)]: 1}), cost_map
 
 
-def variants(net, initial_marking, final_marking):
+def acyclic_net_variants(net, initial_marking, final_marking):
     """
-    Given an acyclic workflow net, initial and final marking extracts a set of variants (list of event)
+    Given an acyclic accepting Petri net, initial and final marking extracts a set of variants (in form of traces)
     replayable on the net.
+    Warning: this function is based on a marking exploration. If the accepting Petri net contains loops, the method
+    will not work properly as it stops the search if a specific marking has already been encountered.
 
     Parameters
     ----------
@@ -168,23 +214,22 @@ def variants(net, initial_marking, final_marking):
 
     Returns
     -------
-    variants: :class:`list` List of variants replayable in the net.
+    variants: :class:`list` List of variants - in the form of Trace objects - obtainable executing the net
 
     """
-    active = [(initial_marking, [])]
+    active = [(initial_marking, Trace())]
     visited = []
-    this_variants = []
-    for i in range(10000000):
-        if not active:
-            break
+    variants = []
+    while active:
         curr_couple = active.pop(0)
         en_tr = petri.semantics.enabled_transitions(net, curr_couple[0])
         for t in en_tr:
-            next_activitylist = list(curr_couple[1])
-            next_activitylist.append(repr(t))
+            next_activitylist = deepcopy(curr_couple[1])
+            if t.label is not None:
+                next_activitylist.append(Event({'concept:name': repr(t)}))
             next_couple = (petri.semantics.execute(t, net, curr_couple[0]), next_activitylist)
             if hash(next_couple[0]) == hash(final_marking):
-                this_variants.append(next_couple[1])
+                variants.append(next_couple[1])
             else:
                 # If the next marking hash is not in visited, if the next marking+partial trace itself is
                 # not already in active and if the next marking+partial trace is different from the
@@ -194,7 +239,7 @@ def variants(net, initial_marking, final_marking):
                         hash(curr_couple[0]) != hash(next_couple[0]) or curr_couple[1] != next_couple[1]):
                     active.append(next_couple)
         visited.append(hash(curr_couple[0]))
-    return this_variants
+    return variants
 
 
 def get_transition_by_name(net, transition_name):
@@ -244,6 +289,74 @@ def get_cycles_petri_net_places(net):
     return cycles_places
 
 
+def get_cycles_petri_net_transitions(net):
+    """
+    Get the cycles of a Petri net (returning only list of transitions belonging to the cycle)
+
+    Parameters
+    -------------
+    net
+        Petri net
+
+    Returns
+    -------------
+    cycles
+        Cycles (transitions) of the Petri net
+    """
+    graph, inv_dictionary = create_networkx_directed_graph(net)
+    cycles = nx.simple_cycles(graph)
+    cycles_trans = []
+    for cycle in cycles:
+        cycles_trans.append([])
+        for el in cycle:
+            if el in inv_dictionary and type(inv_dictionary[el]) is petri.petrinet.PetriNet.Transition:
+                cycles_trans[-1].append(inv_dictionary[el])
+    return cycles_trans
+
+
+def decorate_places_preset_trans(net):
+    """
+    Decorate places with information useful for the replay
+
+    Parameters
+    -------------
+    net
+        Petri net
+    """
+    for place in net.places:
+        place.ass_trans = set()
+
+    for trans in net.transitions:
+        for place in trans.sub_marking:
+            place.ass_trans.add(trans)
+
+
+def decorate_transitions_prepostset(net):
+    """
+    Decorate transitions with sub and addition markings
+
+    Parameters
+    -------------
+    net
+        Petri net
+    """
+    from pm4py.objects.petri.petrinet import Marking
+    for trans in net.transitions:
+        sub_marking = Marking()
+        add_marking = Marking()
+
+        for arc in trans.in_arcs:
+            sub_marking[arc.source] = arc.weight
+            add_marking[arc.source] = -arc.weight
+        for arc in trans.out_arcs:
+            if arc.target in add_marking:
+                add_marking[arc.target] = arc.weight + add_marking[arc.target]
+            else:
+                add_marking[arc.target] = arc.weight
+        trans.sub_marking = sub_marking
+        trans.add_marking = add_marking
+
+
 def get_strongly_connected_subnets(net):
     """
     Get the strongly connected components subnets in the Petri net
@@ -286,7 +399,8 @@ def get_strongly_connected_subnets(net):
     return strongly_connected_subnets
 
 
-def get_places_shortest_path(net, place_to_populate, current_place, places_shortest_path, actual_list, rec_depth):
+def get_places_shortest_path(net, place_to_populate, current_place, places_shortest_path, actual_list, rec_depth,
+                             max_rec_depth):
     """
     Get shortest path between places lead by hidden transitions
 
@@ -304,9 +418,10 @@ def get_places_shortest_path(net, place_to_populate, current_place, places_short
         Actual list of transitions to enable
     rec_depth
         Recursion depth
+    max_rec_depth
+        Maximum recursion depth
     """
-    MAX_REC_DEPTH = 18
-    if rec_depth > MAX_REC_DEPTH:
+    if rec_depth > max_rec_depth:
         return places_shortest_path
     if place_to_populate not in places_shortest_path:
         places_shortest_path[place_to_populate] = {}
@@ -320,11 +435,11 @@ def get_places_shortest_path(net, place_to_populate, current_place, places_short
                     places_shortest_path[place_to_populate][p2.target] = copy(new_actual_list)
                     places_shortest_path = get_places_shortest_path(net, place_to_populate, p2.target,
                                                                     places_shortest_path, new_actual_list,
-                                                                    rec_depth + 1)
+                                                                    rec_depth + 1, max_rec_depth)
     return places_shortest_path
 
 
-def get_places_shortest_path_by_hidden(net):
+def get_places_shortest_path_by_hidden(net, max_rec_depth):
     """
     Get shortest path between places lead by hidden transitions
 
@@ -332,10 +447,12 @@ def get_places_shortest_path_by_hidden(net):
     ----------
     net
         Petri net
+    max_rec_depth
+        Maximum recursion depth
     """
     places_shortest_path = {}
     for p in net.places:
-        places_shortest_path = get_places_shortest_path(net, p, p, places_shortest_path, [], 0)
+        places_shortest_path = get_places_shortest_path(net, p, p, places_shortest_path, [], 0, max_rec_depth)
     return places_shortest_path
 
 
@@ -397,7 +514,7 @@ def remove_unconnected_components(net):
 
 
 def get_s_components_from_petri(net, im, fm, rec_depth=0, curr_s_comp=None, visited_places=None,
-                                list_s_components=None):
+                                list_s_components=None, max_rec_depth=6):
     """
     Gets the S-components from a Petri net
 
@@ -415,13 +532,14 @@ def get_s_components_from_petri(net, im, fm, rec_depth=0, curr_s_comp=None, visi
         Visited places
     list_s_components
         List of S-components
+    max_rec_depth
+        Maximum recursion depth
 
     Returns
     --------------
     s_components
         List of S-components
     """
-    MAX_REC_DEPTH = 6
     if list_s_components is None:
         list_s_components = []
     if len(im) > 1 or len(fm) > 1:
@@ -434,7 +552,7 @@ def get_s_components_from_petri(net, im, fm, rec_depth=0, curr_s_comp=None, visi
     if visited_places is None:
         visited_places = []
     something_changed = True
-    while something_changed and rec_depth < MAX_REC_DEPTH:
+    while something_changed and rec_depth < max_rec_depth:
         something_changed = False
         places_to_visit = sorted(list(set(curr_s_comp[len(visited_places):])), key=lambda x: len(x.out_arcs),
                                  reverse=True)
@@ -458,7 +576,8 @@ def get_s_components_from_petri(net, im, fm, rec_depth=0, curr_s_comp=None, visi
                             list_s_components = get_s_components_from_petri(net, im, fm, rec_depth=rec_depth + 1,
                                                                             curr_s_comp=new_curr_s_comp,
                                                                             visited_places=new_visited_places,
-                                                                            list_s_components=list_s_components)
+                                                                            list_s_components=list_s_components,
+                                                                            max_rec_depth=max_rec_depth)
 
     if not set([place.name for place in curr_s_comp]) in list_s_components:
         list_s_components.append(set([place.name for place in curr_s_comp]))
