@@ -1,13 +1,14 @@
 import pandas as pd
 
-from pm4py.algo.filtering.common.filtering_constants import CASE_CONCEPT_NAME
-from pm4py.objects.log.util import xes
-from pm4py.objects.log.util.xes import DEFAULT_TIMESTAMP_KEY
+from pm4py.util import xes_constants as xes
+from pm4py.util.xes_constants import DEFAULT_TIMESTAMP_KEY
 from pm4py.statistics.traces.common import case_duration as case_duration_commons
 from pm4py.util.constants import PARAMETER_CONSTANT_ACTIVITY_KEY
 from pm4py.util.constants import PARAMETER_CONSTANT_CASEID_KEY
 from pm4py.util.constants import PARAMETER_CONSTANT_TIMESTAMP_KEY
+from pm4py.util.constants import CASE_CONCEPT_NAME
 
+import pandas as pd
 
 def get_variant_statistics(df, parameters=None):
     """
@@ -38,7 +39,7 @@ def get_variant_statistics(df, parameters=None):
                                                                                                 parameters=parameters)
     variants_df = variants_df.reset_index()
     variants_list = variants_df.groupby("variant").agg("count").reset_index().to_dict('records')
-    variants_list = sorted(variants_list, key=lambda x: x[case_id_glue], reverse=True)
+    variants_list = sorted(variants_list, key=lambda x: (x[case_id_glue], x["variant"]), reverse=True)
     if max_variants_to_return:
         variants_list = variants_list[:min(len(variants_list), max_variants_to_return)]
     return variants_list
@@ -75,7 +76,7 @@ def get_variant_statistics_with_case_duration(df, parameters=None):
     variants_df = variants_df.reset_index()
     variants_list = variants_df.groupby("variant").agg({"caseDuration": "mean", "count": "sum"}).reset_index().to_dict(
         'records')
-    variants_list = sorted(variants_list, key=lambda x: x["count"], reverse=True)
+    variants_list = sorted(variants_list, key=lambda x: (x["count"], x["variant"]), reverse=True)
     if max_variants_to_return:
         variants_list = variants_list[:min(len(variants_list), max_variants_to_return)]
     return variants_list
@@ -154,6 +155,7 @@ def get_cases_description(df, parameters=None):
     max_ret_cases = parameters["max_ret_cases"] if "max_ret_cases" in parameters else None
 
     grouped_df = df[[case_id_glue, timestamp_key]].groupby(df[case_id_glue])
+    # grouped_df = df[[case_id_glue, timestamp_key]].groupby(df[case_id_glue])
     first_eve_df = grouped_df.first()
     last_eve_df = grouped_df.last()
     del grouped_df
@@ -203,7 +205,11 @@ def get_variants_df(df, parameters=None):
     activity_key = parameters[
         PARAMETER_CONSTANT_ACTIVITY_KEY] if PARAMETER_CONSTANT_ACTIVITY_KEY in parameters else xes.DEFAULT_NAME_KEY
 
-    return df.groupby(case_id_glue)[activity_key].agg({'variant': lambda col: ','.join(col)})
+    new_df = df.groupby(case_id_glue)[activity_key].agg(lambda col: ",".join(pd.Series.to_list(col))).to_frame()
+    new_cols = list(new_df.columns)
+    new_df = new_df.rename(columns={new_cols[0]: "variant"})
+
+    return new_df
 
 
 def get_variants_df_with_case_duration(df, parameters=None):
@@ -235,7 +241,11 @@ def get_variants_df_with_case_duration(df, parameters=None):
     timestamp_key = parameters[
         PARAMETER_CONSTANT_TIMESTAMP_KEY] if PARAMETER_CONSTANT_TIMESTAMP_KEY in parameters else xes.DEFAULT_TIMESTAMP_KEY
     grouped_df = df[[case_id_glue, timestamp_key, activity_key]].groupby(df[case_id_glue])
-    df1 = grouped_df[activity_key].agg({'variant': lambda col: ','.join(col)})
+
+    df1 = grouped_df[activity_key].agg(lambda col: ",".join(pd.Series.to_list(col))).to_frame()
+    new_cols = list(df1.columns)
+    df1 = df1.rename(columns={new_cols[0]: "variant"})
+
     first_eve_df = grouped_df.first()
     last_eve_df = grouped_df.last()
     del grouped_df
